@@ -42,9 +42,10 @@
 #define _CF_BODYFILE	'b'	/* alternate message file for new body */
 #define _CF_SENDER	's'	/* sender triple (channel, host, user) */
 #define _CF_RECIPIENT	'r'	/* recipient n-tuple, n >= 3 */
-#define _CF_XORECIPIENT	'R'	/* one of XOR set of recipient n-tuples */
+#define _CF_DSNRECIPIENT 'R'	/* DSN view of the recipient addr */
+#define _CF_XORECIPIENT	'X'	/* one of XOR set of recipient n-tuples */
 #define _CF_RCPTNOTARY	'N'	/* DSN parameters for previous recipient */
-#define _CF_DSNENVID	'n'	/* DNS 'MAIL FROM<> ENVID=XXXX' data */
+#define _CF_DSNENVID	'n'	/* DSN 'MAIL FROM<> ENVID=XXXX' data */
 #define _CF_ERRORADDR	'e'	/* return address for error messages */
 #define _CF_DIAGNOSTIC	'd'	/* diagnostic message for ctlfile offset */
 #define _CF_MSGHEADERS	'm'	/* message header for preceeding recipients */
@@ -52,6 +53,7 @@
 #define _CF_OBSOLETES	'x'	/* message id of message obsoleted by this */
 #define _CF_VERBOSE	'v'	/* log file name for verbose log (mail -v) */
 #define	_CF_TURNME	'T'	/* trigger scheduler to attempt delivery now */
+#define _CF_RCVFROM	'F'	/* Where-from we are coming ? */
 
 /* The following characters may appear in the second column after most _CF_* */
 #define	_CFTAG_NORMAL	' '	/* what the router sets it to be */
