@@ -2,6 +2,9 @@
  *	Copyright 1990 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
  */
+/*
+ *	A plenty of changes, copyright Matti Aarnio 1990-1995
+ */
 
 #include "libsupport.h"
 
@@ -118,8 +121,8 @@ extern int	       ctlsticky __((char *spec_host,char *addr_host));
 
 /* diagnostic.c: */
 extern char           *notaryacct __((int rc, char *okstr));
-extern void 	       notaryreport __((char*,char*,char*));
-extern void	       notarystatsave __((char *smtpstatline));
+		/* NOTARY: addres / action / status / diagnostic  */
+extern void 	       notaryreport __((char*,char*,char*,char*));
 extern void	       diagnostic __((/* struct rcpt *, int, int, char *,... */));
 
 /* dnsgetrr.c */
@@ -170,6 +173,10 @@ extern int downgrade_charset __((struct rcpt *rp, FILE *verboselog));
 extern void downgrade_headers __((struct rcpt *rp, int downgrade, FILE *verboselog));
 extern int qp_to_8bit __((struct rcpt *rp));
 
+/* mime2headers.c */
+extern int headers_to_mime2 __((struct rcpt *rp, FILE *verboselog));
+
+
 /* writeheaders.c: */
 extern int writeheaders __((struct rcpt *rp, FILE *fp, char *newline, int use_cvt, int maxwidth));
 
