@@ -209,12 +209,10 @@ _unfold(start, end, t)
  * for efficiency this was changed to avoid function call overhead. The tokens
  * returned are classified by type (TokenType enum class).
  */
-
-struct token *
-scan822(cpp, n, c1, c2, allowcomments, tlistp)
+struct token * scan822(cpp, n, c1, c2, allowcomments, tlistp)
 	u_char	**cpp;			/* pointer to pointer to text */
 	long	n;			/* number of characters to scan */
-	u_char	c1, c2;			/* temporary specials */
+	int	c1, c2;			/* temporary specials */
 	int allowcomments;		/* #prefix tokens are comments to EOT */
 	struct token **tlistp;		/* continuation line tokens if any */
 {
