@@ -22,14 +22,12 @@ static void fopenerr(name,msg,dumpfps,oerrno)
 char *name, *msg;
 int dumpfps, oerrno;
 {
-	extern char *sys_errlist[];
-
 	if (!fopenlog) abort();
 
 	fprintf(fopenlog,"__%s() : %s",name,msg);
 
 	if (oerrno != 0)
-	  fprintf(fopenlog,", errno=%d (%s)",oerrno,sys_errlist[oerrno]);
+	  fprintf(fopenlog,", errno=%d (%s)",oerrno, strerror(oerrno));
 
 	if (dumpfps) {
 	  int i;
