@@ -343,9 +343,9 @@ mail_close(fp)
 	 * data without being told about it.
 	 */
 	if (fflush(fp) == EOF
-#ifdef	NFSFSYNC
+/* #ifdef NFSFSYNC */ /* This is propably ALWAYS a good idea */
 	    || fsync(fileno(fp)) < 0
-#endif	/* NFSFSYNC */
+/* #endif */	/* NFSFSYNC */
 	    || fclose(fp) == EOF) {
 		mail_free(message);
 		errno = EIO;
