@@ -43,13 +43,13 @@
 #endif
 #endif /* USE_SUNGETMNT */
 
-#ifdef	USE_MNTENT	/* Solaris, __svr4__ */
+#ifdef	USE_SVR4MNTENT	/* Solaris, __svr4__ */
 #include <sys/mnttab.h>
 #include <sys/mntent.h>
 #define	MNTTYPE	struct mnttab
-#endif /* USE_MNTENT */
+#endif /* USE_SVR4MNTENT */
 
-#ifdef USE_LINUXMNTENT
+#ifdef USE_POSIXMNTENT
 #include <mntent.h>
 #define MNTTYPE struct mntent
 #endif
@@ -179,7 +179,7 @@ getmntpt(file, dir)
 #endif
 #endif	/* USE_NFS */
 
-#ifdef	USE_MNTENT /* Solaris / SysVR4 */
+#ifdef	USE_SVR4MNTENT /* Solaris / SysVR4 */
 static char *
 getmntpt(file, dir)
 	char	*file, **dir;
@@ -212,10 +212,10 @@ getmntpt(file, dir)
 	fclose(mntp);
 	return NULL;
 }
-#endif /* USE_MNTENT */
+#endif /* USE_SVR4MNTENT */
 
 
-#ifdef USE_LINUXMNTENT
+#ifdef USE_POSIXMNTENT
 static char *
 getmntpt(file, dir)
 	char	*file, **dir;
@@ -247,7 +247,7 @@ getmntpt(file, dir)
 	endmntent(mntp);
 	return NULL;
 }
-#endif	/* USE_LINUXMNTENT */
+#endif	/* USE_POSIXMNTENT */
 
 
 
