@@ -684,7 +684,10 @@ runcommand(c, pc, retcodep)
 
 	/* run the command and undo any temporary variables */
 	if (retcodep != NULL) {
-		*retcodep = execute(c, pc, *retcodep);
+		char *name = "<builtin cmd>";
+		if (c->shcmdp != NULL)
+		  name = c->shcmdp->name;
+		*retcodep = execute(c, pc, *retcodep, name);
 		if (*retcodep != 0 && isset('e'))
 			trapexit(*retcodep);
 	}
@@ -842,7 +845,7 @@ xundefun(spl)
 	tregexp **trep, **trepstart;
 #endif	/* MAILER */
 	
-	if (sfdp = (struct sslfuncdef *)spl->data) {
+	if ((sfdp = (struct sslfuncdef *)spl->data)) {
 		psfdp = &(sfdp->tabledesc->functions);
 		for (sfdp = *psfdp;
 		     sfdp != NULL; psfdp = &sfdp->next, sfdp = *psfdp) {
