@@ -107,9 +107,10 @@ sioputs(s)
 	if (!s)
 		return EOF;
 	siofputs(s, stdout);
-	if (--siop->sb_cnt <= 0)
+	if ((--siop->sb_cnt) <= 0)
 	  siomore(siop);
-	*siop->sb_ptr++ = '\n';
+	*siop->sb_ptr = '\n';
+	siop->sb_ptr++;
 	return 0;
 }
 
@@ -123,7 +124,7 @@ siofputs(s, fp)
 	if (!s)
 		return EOF;
 	while (*s != '\0') {
-	  if (--siop->sb_cnt <= 0)
+	  if ((--siop->sb_cnt) <= 0)
 	    siomore(siop);
 	  *siop->sb_ptr++ = (u_char)*s++;
 	}
@@ -205,7 +206,7 @@ siofread(b, s, n, fp)
 	n = s;
 	/* assert siop != NULL */
 	while ((s > 0) &&
-	       (siop->sb_ptr + siop->sb_cnt < siop->sb_base + siop->sb_bufsiz))
+	       ((siop->sb_ptr + siop->sb_cnt) < (siop->sb_base + siop->sb_bufsiz)))
 	  --s, *b++ = *siop->sb_ptr++;
 	return n-s;
 }
@@ -223,7 +224,7 @@ siofwrite(b, s, n, fp)
 	s *= n;
 	n = s;
 	while (s-- > 0) {
-	  if (--siop->sb_cnt <= 0)
+	  if ((--siop->sb_cnt) <= 0)
 	    siomore(siop);
 	  *siop->sb_ptr++ = (u_char)*b++;
 	}
@@ -233,7 +234,7 @@ siofwrite(b, s, n, fp)
 
 #define	CLICK	120		/* this should cover most addresses/lists */
 
-int
+volatile int
 siomore(siop)
 	struct siobuf *siop;
 {
@@ -306,9 +307,20 @@ STATIC char sccsid[] = "@(#)doprnt.c	5.35 (Berkeley) 6/27/88";
 
 #define	BUF		(MAXEXP+MAXFRACT+1)	/* + decimal point */
 
-#define	PUTC(ch)	(void) ((--siop->sb_cnt < 0 ? siomore(siop) : 0), \
-				(int)(*siop->sb_ptr++ = (u_char)(ch)))
-
+#if 1
+static void sio_PUTC(ch,siop)
+register int ch;
+register struct siobuf *siop;
+{
+	if ((--siop->sb_cnt) <= 0)
+	  siomore(siop);
+	*(siop->sb_ptr++) = (u_char) ch;
+}
+#define PUTC(ch) sio_PUTC(ch,siop)
+#else
+#define	PUTC(ch)	(void) ((((--siop->sb_cnt) <= 0) ? siomore(siop) : 0), \
+				(int)(*(siop->sb_ptr++) = (u_char)(ch)))
+#endif
 #define	ARG() \
 	_ulong = flags&LONGINT ? va_arg(argp, long) : \
 	    flags&SHORTINT ? va_arg(argp, short) : va_arg(argp, int);
@@ -643,7 +655,7 @@ pforw:
 				PUTC('0');
 
 			/* the string or number proper */
-			if (siop->sb_cnt - (n = size) >= 0) {
+			if (siop->sb_cnt - (n = size) > 0) {
 				siop->sb_cnt -= n;
 				memcpy((char *)siop->sb_ptr, t, n);
 				siop->sb_ptr += n;
