@@ -137,11 +137,13 @@ int		D_bind = 0;
 int		D_resolv = 0;
 int		D_alloc = 0;
 int		D_regnarrate = 0;
+extern int	D_rfc822;
 
 static struct debugind {
 	char	*name;
 	int	*indicator;
 } buggers[] = {
+	{	"rfc822",		&D_rfc822	},
 	{	"sequencer",		&D_sequencer	},
 	{	"rewrite",		&D_hdr_rewrite	},
 	{	"router",		&D_router	},
