@@ -104,7 +104,7 @@ add_core(sip, value)
 	stickymem = oval;
 
 	if ((spl = sp_lookup(symbol(sip->key), db)) == NULL)
-		sp_install(symbol(sip->key), value, 0, db);
+		sp_install(symbol(sip->key), (void*)value, 0, db);
 	else {
 		icfreedata(spl);
 		spl->data = (u_char *)value;
