@@ -35,7 +35,8 @@ search_bin(sip)
 	register unsigned char *cp, *s;
 	register int c;
 	long int top, bot, mid;
-	int i, retry, symid;
+	int i, retry;
+	spkey_t symid;
 	struct conscell *tmp;
 	struct spblk *spl;
 	char buf[BUFSIZ];
