@@ -50,8 +50,9 @@ extern struct headerinfo *find_header();
 extern struct headerinfo *find_header();
 extern struct headerinfo envelope_hdrs[];
 extern struct headerinfo mandatory_hdrs[];
-extern struct token * scan822 __((u_char *cpp, long n, u_char c1, u_char c2,
-				  int allowcomments, struct token *tlistp));
+extern struct token * scan822 __((u_char **cpp, long n, int c1, int c2,
+				  int allowcomments, struct token **tlistp));
+extern char *rfc822date __((time_t *tim));
 extern union misc parse822();
 extern void	dumpHeader();
 extern void	hdr_print();
