@@ -414,15 +414,18 @@ int	isRecpntAddr = 0;
 
 #define FindEnvelope(X)	\
 	for (h = e->e_eHeaders; h != NULL; h = h->h_next) \
-		if (h->h_descriptor->class == X) \
-			break;
+	    if (h->h_descriptor->class == X) \
+		break;
 
-#define	FindHeader(X) \
+#define RESENT_SIZE (sizeof "resent-" - 1)
+
+#define	FindHeader(X,RESENT) \
 	for (h = e->e_headers; h != NULL; h = h->h_next) \
-		if ((e->e_resent == 0 || h->h_descriptor->class == Resent) \
-		    && h->h_descriptor->hdr_name != NULL \
-		    && STREQ(h->h_descriptor->hdr_name + e->e_resent, X)) \
-			break;
+	    if ((RESENT ? (h->h_descriptor->class == Resent) : \
+		          (h->h_descriptor->class != Resent)   ) \
+		&& h->h_descriptor->hdr_name != NULL \
+		&& STREQ(h->h_descriptor->hdr_name+(RESENT?RESENT_SIZE:0), X))\
+		    break;
 
 /* we insert the new header just before this point */
 #define InsertHeader(IH, EXPR) \
@@ -508,7 +511,7 @@ erraddress(e)
 		FindEnvelope(eFrom);	/* should never be NULL */
 		/* but might still be problematic */
 		if (h != NULL && h->h_stamp == BadHeader)
-			FindHeader("from");
+		  FindHeader("from",e->e_resent);
 	} else
 		h = best;
 	if (h != NULL && h->h_stamp == BadHeader)
@@ -1068,13 +1071,10 @@ sequencer(e, file)
 	e->e_resent = 0;
 	for (h = e->e_headers; h != NULL; h = h->h_next)
 		if (h->h_descriptor->class == Resent) {
-			e->e_resent = sizeof "resent-" - 1;
+			e->e_resent = 1;
 			break;
 		}
-	dprintf("It is%s...\n", e->e_resent > 0 ? "" : "n't");
-/* Actually it may be resent, but we (people at UTU.FI) don't like all
-   that it causes latter on.. */
-	e->e_resent = 0;
+	dprintf("It is%s...\n", e->e_resent ? "" : "n't");
 
 	dprintf("Sender authentication\n");
 	e->e_trusted = isgoodguy(e->e_statbuf.st_uid);
@@ -1098,7 +1098,10 @@ sequencer(e, file)
 
 	if (myhostname) {	/* we care about message id's */
 		dprintf("Make sure Message-Id exists, for loop control\n");
-		FindHeader("message-id");
+		/* it may be  message-id, or resent-message-id */
+		FindHeader("message-id",e->e_resent);
+		if (h == NULL) /* Ok, wasn't, how about resent-message-id ? */
+		  FindHeader("message-id",!e->e_resent);
 		if (h == NULL) {
 			/* the time used must be the same as for trace header */
 			InsertHeader(h, mkMessageId(e, e->e_nowtime));
@@ -1222,14 +1225,16 @@ sequencer(e, file)
 			FindEnvelope(eFrom);
 			/* X: assert h != NULL */
 			if (h == NULL || h->h_stamp == BadHeader) {
-				/* XX: perhaps should be other way around? */
-				FindHeader("sender");
-				if (h == NULL || h->h_stamp == BadHeader)
-					FindHeader("from");
-				if (h == NULL || h->h_stamp == BadHeader)
-				  h = mkSender(e,
-					       uidpwnam(e->e_statbuf.st_uid),
-					       0);
+			  /* XX: perhaps should be other way around? */
+			  FindHeader("sender",e->e_resent);
+			  if (h == NULL || h->h_stamp == BadHeader)
+			    FindHeader("from",e->e_resent);
+			  if (h == NULL || h->h_stamp == BadHeader)
+			    FindHeader("sender",!e->e_resent);
+			  if (h == NULL || h->h_stamp == BadHeader)
+			    FindHeader("from",!e->e_resent);
+			  if (h == NULL || h->h_stamp == BadHeader)
+			    h = mkSender(e,uidpwnam(e->e_statbuf.st_uid),0);
 			}
 			if (h == NULL)
 				abort();
@@ -1260,9 +1265,9 @@ sequencer(e, file)
 	} else {
 		dprintf("We know sender is local and one of the peons\n");
 		def_uid = e->e_statbuf.st_uid;
-		FindHeader("from");
+		FindHeader("from",e->e_resent);
 		nh = h;
-		FindHeader("sender");
+		FindHeader("sender",e->e_resent);
 		if (h != NULL && nh == NULL) {
 			dprintf("A Sender w/o a From is bad; fixed\n");
 			h->h_descriptor = senderDesc();
@@ -1281,24 +1286,21 @@ sequencer(e, file)
 		} else if (nh != NULL && nh->h_contents.a != NULL) {
 			/* only a From: was given */
 			if (!thesender(e, nh->h_contents.a)) {
-				/* but it is fake, so add a Sender: */
-				dprintf("The From: is not the sender\n");
-				if (h != NULL) {
-					/* use our empty Sender: */
-					ph = mkSender(e,
-						uidpwnam(e->e_statbuf.st_uid),
-						0);
-					h->h_contents.a = ph->h_contents.a;
-					h = NULL;
-				} else
-					h = nh;
+			  /* but it is fake, so add a Sender: */
+			  dprintf("The From: is not the sender\n");
+			  if (h != NULL) {
+			    /* use our empty Sender: */
+			    ph = mkSender(e,uidpwnam(e->e_statbuf.st_uid),0);
+			    h->h_contents.a = ph->h_contents.a;
+			    h = NULL;
+			  } else
+			    h = nh;
 			} else
-				h = NULL;
+			  h = NULL;
 		}
 		if (h != NULL) {
-			InsertHeader(h,
-				mkSender(e, uidpwnam(e->e_statbuf.st_uid), 0));
-			set_pname(e, nh, "Sender");
+		  InsertHeader(h,mkSender(e, uidpwnam(e->e_statbuf.st_uid),0));
+		  set_pname(e, nh, "Sender");
 		}
 	}
 	if (QCHANNEL(e->e_from_trusted) == NULL)
@@ -1344,17 +1346,17 @@ sequencer(e, file)
 	dprintf("Nuke Bcc headers if any\n");
 	oh = NULL;
 	for (h = e->e_headers; h != NULL; oh = h, h = h->h_next) {
-		if (h->h_descriptor->hdr_name == NULL)
-			continue;
-		if ((h->h_descriptor->class == normal
-		     && CISTREQ(h->h_descriptor->hdr_name,"bcc"))
-		    || (h->h_descriptor->class == Resent
-			&& CISTREQ(h->h_descriptor->hdr_name, "resent-bcc"))) {
-			if (oh == NULL)
-				e->e_headers = h->h_next;
-			else
-				oh->h_next = h->h_next;
-		}
+	  if (h->h_descriptor->hdr_name == NULL)
+	    continue;
+	  if ((h->h_descriptor->class == normal
+	       && CISTREQ(h->h_descriptor->hdr_name,"bcc"))
+	      || (h->h_descriptor->class == Resent
+		  && CISTREQ(h->h_descriptor->hdr_name, "resent-bcc"))) {
+	    if (oh == NULL)
+	      e->e_headers = h->h_next;
+	    else
+	      oh->h_next = h->h_next;
+	  }
 	}
 
 	/*
@@ -1381,7 +1383,7 @@ sequencer(e, file)
 		}
 	}
 	dprintf("Make sure a From: line exists...");
-	FindHeader("from");
+	FindHeader("from",e->e_resent);
 	if (h == NULL) {
 		FindEnvelope(eFrom);
 		if (h == NULL) {	/* panic... can't happen... */
@@ -1392,25 +1394,28 @@ sequencer(e, file)
 			nh = (struct header *)tmalloc(sizeof (struct header));
 			*nh = *h;
 			set_pname(e, nh, "From");
-			FindHeader("to");
+			FindHeader("to",e->e_resent);
 			if (h == NULL) {
-				for (h = e->e_headers; h != NULL; h = h->h_next)
-					if (CISTREQ(h->h_pname, "subject"))
-						break;
+			  for (h = e->e_headers; h != NULL; h = h->h_next)
+			    if (CISTREQ(h->h_pname, "subject"))
+			      break;
 			}
 			InsertHeader(h, nh);
 			dprintf(" nope (added)!\n");
 		}
 	} else
 		dprintf(" yup!\n");
-	FindHeader("to");
+	/* Any 'To:' in the headers ? */
+	FindHeader("to",e->e_resent);
+	if (h == NULL)
+	  FindHeader("to",!e->e_resent);
 	if (h == NULL) {
 		dprintf("Insert the To: header lines\n");
 		for (h = e->e_headers; h != NULL; h = h->h_next)
 			if (CISTREQ(h->h_pname, "subject"))
 				break;
 		if (h == NULL) {
-			FindHeader("from");
+			FindHeader("from",e->e_resent);
 			if (h != NULL)
 				h = h->h_next;
 		}
@@ -1607,12 +1612,12 @@ sequencer(e, file)
  
 	isSenderAddr = isRecpntAddr = 0;
 	dprintf("Make sure Date, From, To, are in the header\n");
-	FindHeader("date");
+	FindHeader("date",e->e_resent);
 	if (h == NULL)
-		InsertHeader(h, mkDate(e->e_resent, e->e_statbuf.st_mtime));
+	  InsertHeader(h, mkDate(e->e_resent, e->e_statbuf.st_mtime));
 
 	if (rwhead == NULL)
-		return PERR_NORECIPIENTS;
+	  return PERR_NORECIPIENTS;
 
 	dprintf("Emit specification to the transport system\n");
 #ifdef	USE_ALLOCA
