@@ -18,6 +18,7 @@
 
 
 struct headerinfo nullhdr = { 0, nilHeaderSemantics, nilUserType, normal };
+int D_rfc822 = 0; /* Debug traceing */
 
 /*
  * Apply RFC822 scanner and parser to the list of Line tokens passed as args.
@@ -108,12 +109,14 @@ hdr_scanparse(e, h, commentflag)
 		}
 	}
 	*prev_tp = NULL;
+
 	/* parse it */
 	if (hd->semantics == nilHeaderSemantics)
 		retval.t = tlist;
 	else
 		retval = parse822(hd->semantics,
-				  &tlist, &(e->e_localtime), (FILE *)NULL);
+				  &tlist, &(e->e_localtime),
+				  D_rfc822 ? stderr : NULL);
 	return retval;
 }
 
@@ -343,7 +346,7 @@ mkDate(isresent, unixtime)
 	h->h_next = 0;
 	h->h_contents.d = unixtime;
 	cp = rfc822date(&(h->h_contents.d));
-	s = cp + strlen(cp) -1;
+	s = cp + strlen((char*)cp) -1;
 	if (*cp != 0 && *s == '\n') *s = 0;
 	h->h_lines = makeToken(cp, strlen((char *)cp));
 	h->h_lines->t_type = Line;
