@@ -32,7 +32,9 @@ STATIC TableOperation	operation;	/* the current operation */
 
 /* convenience */
 STATIC struct address nullAddr = { NULL, NULL, newAddress, 0, NULL, NULL };
+#ifdef ATHACK
 STATIC struct token atsigntoken = { (u_char *)"@", 1, Special, NULL };
+#endif
 
 /* Tracing Control */
 STATIC FILE	*tracefp;		/* if non-null, trace output here */
@@ -252,7 +254,10 @@ next:
 					nextInputToken = tPeriod;
 					break;
 				case ':':
-					nextInputToken = tColon;
+					if ((*tlistp)->t_pname[1] == ':')
+					  nextInputToken = tDoubleColon;
+					else
+					  nextInputToken = tColon;
 					break;
 				case '<':
 					nextInputToken = tLeftAngle;
@@ -384,6 +389,7 @@ Input822TokenName(in)
 	case tSemicolon:	return "tSemicolon";
 	case tComma:		return "tComma";
 	case tPeriod:		return "tPeriod";
+	case tDoubleColon:	return "tDoubleColon";
 	case tColon:		return "tColon";
 	case tLeftAngle:	return "tLeftAngle";
 	case tRightAngle:	return "tRightAngle";
@@ -697,6 +703,12 @@ parse822(entry, tlistp, ltmp, tfp)
 	returnType = rAddress;
 	currentTokenType = cAddress;
 
+	if (tracefp) {
+	  sslPointer = ((int)entry) + 1;
+	  operation = -1;
+	  SslTrace();
+	}
+
 	/* Initialize Input/Output */
 	AcceptInputToken(tlistp, ap);
 	if (nextInputToken == tEndOfHeader) {
