@@ -441,7 +441,7 @@ router(a, uid, type)
 	 * We expect router to either return
 	 * (local - user attributes) or (((local - user attributes)))
 	 */
-	if (LIST(car(s_value))) {
+	if (car(s_value) && LIST(car(s_value))) {
 		if (!LIST(caar(s_value)) || !STRING(caaar(s_value))) {
 			fprintf(stderr,
 				"%s: '%s' returned invalid 2-level list: ",
