@@ -2,6 +2,10 @@
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
  */
+/*
+ *	Lots of modifications (new guts, more or less..) by
+ *	Matti Aarnio <mea@nic.funet.fi>  (copyright) 1992-1995
+ */
 
 #include "hostenv.h"
 #include <stdio.h>
@@ -25,10 +29,6 @@
 #include <sys/file.h>
 #endif	/* USE_INET */
 
-#ifdef sgi
-flock() { return 0; }
-#endif
-
 #ifdef	MALLOC_TRACE
 struct conshell *envarlist = NULL;
 #endif	/* MALLOC_TRACE */
@@ -57,7 +57,7 @@ extern void report();
 char	*progname;
 char	*postoffice;
 
-int	debug, verbose, summary, user, status, onlyuser, nonlocal;
+int	debug, verbose, summary, user, status, onlyuser, nonlocal, schedq;
 
 time_t	now;
 
@@ -76,7 +76,7 @@ main(argc, argv)
 	short port = 0;
 	struct in_addr naddr;
 	char *host = NULL;
-	struct hostent *hp, he;
+	struct hostent *hp = NULL, he;
 	struct sockaddr_in sad;
 	struct servent *serv = NULL;
 #else	/* !AF_INET */
@@ -88,7 +88,7 @@ main(argc, argv)
 
 	progname = argv[0];
 	verbose = debug = errflg = status = user = onlyuser = summary = 0;
-	while ((c = getopt(argc, argv, "dip:r:stu:vVS")) != EOF) {
+	while ((c = getopt(argc, argv, "dip:r:stu:vVSQ")) != EOF) {
 	  switch (c) {
 	  case 'd':
 	    ++debug;
@@ -139,6 +139,9 @@ main(argc, argv)
 	  case 'S':
 	    ++summary;
 	    break;
+	  case 'Q':
+	    ++schedq;
+	    break;
 	  case 'V':
 	    prversion("mailq");
 	    exit(0);
@@ -350,7 +353,7 @@ docat(file, fd)
 {
 	char buf[BUFSIZ];
 	int n;
-	FILE *fp;
+	FILE *fp = NULL;
 
 	if (fd < 0 && (fp = fopen(file, "r")) == NULL) {
 	  fprintf(stderr, "%s: %s: %s\n", progname, file, strerror(errno));
@@ -464,7 +467,6 @@ checkscheduler()
 {
 	int pid, n, tn, r, sawcore = 0;
 	FILE *fp;
-	struct stat corebuf;
 	struct NDIR_TYPE *dp;
 	DIR *dirp;
 
@@ -478,11 +480,18 @@ checkscheduler()
 	  return;
 	}
 	sawcore = 0;
+	tn = 0;
 	for (dp = readdir(dirp), n = 0; dp != NULL; dp = readdir(dirp)) {
+	  if (dp->d_name[0] == '.' &&
+	      (dp->d_name[1] == 0 || (dp->d_name[1] == '.' &&
+				      dp->d_name[2] == 0)))
+	    continue; /* . and .. */
 	  if (isascii(dp->d_name[0]) && isdigit(dp->d_name[0]))
 	    ++n;
 	  else if (strcmp("core",dp->d_name)==0)
-	    sawcore = 1;
+	    sawcore = 1, ++tn;
+	  else
+	    ++tn;
 	}
 #ifdef	BUGGY_CLOSEDIR
 	/*
@@ -517,7 +526,7 @@ checkscheduler()
 	  break;
 	default:
 	  printf("never started");
-	  if (tn > 0)
+	  if (n > 0)
 	    printf(" \"%s/%s\" polluted", postoffice, SCHEDULERDIR);
 	  break;
 	}
@@ -557,18 +566,19 @@ isalive(pidfile, pidp, fpp)
 #define	LEN_MAGIC_PREAMBLE	(sizeof MAGIC_PREAMBLE - 1)
 #define	VERSION_ID		"zmailer 1.0"
 #define	GETLINE(buf,fp)		\
-	if (fgets(buf, sizeof buf, fp) == NULL) {\
+	if (fgets(buf, bufsize, fp) == NULL) {\
 		fprintf(stderr, "%s: no input from scheduler\n", progname);\
 		buf[0] = '\0';\
 		return 0;\
 	}\
-	{ char *s; if (s = strchr(buf,'\n')) *s = '\0'; }
+	{ char *s; if ((s = strchr(buf,'\n'))) *s = '\0'; }
 				
 
-char	*names[SIZE_L] = { "Vertices:", "Hosts:", "Channels:" };
+char	*names[SIZE_L+2];
 
 #define	L_VERTEX	SIZE_L
-struct sptree *spt_ids[SIZE_L+1];
+#define L_END		SIZE_L+1
+struct sptree *spt_ids[SIZE_L+2];
 
 #define	EQNSTR(a,b)	(!strncmp(a,b,strlen(b)))
 
@@ -583,7 +593,16 @@ parse(fp)
 	register int	i;
 	u_long	list, key;
 	struct spblk *spl;
-	char	buf[8192*32], *ocp; /* XXX: !!! SOMETIMES THERE ARE HELLUVA LONG LINES !! */
+	int bufsize = 8192*32; /* Sometimes the input has LONG lines.. */
+	char  *buf, *ocp;
+
+	names[L_CTLFILE] = "Vertices:";
+	names[L_HOST]    = "Hosts:";
+	names[L_CHANNEL] = "Channels:";
+	names[L_END]     = "End:";
+
+
+	buf = emalloc(bufsize);
 
 	GETLINE(buf,fp);
 	if (!(EQNSTR(buf, MAGIC_PREAMBLE)
@@ -596,16 +615,25 @@ parse(fp)
 	  return 0;
 	list = L_CTLFILE;
 	spt_ids[L_CTLFILE] = sp_init();
-	spt_ids[L_VERTEX] = sp_init();
+	spt_ids[L_VERTEX ] = sp_init();
 	spt_ids[L_CHANNEL] = sp_init();
-	spt_ids[L_HOST] = sp_init();
-	while (fgets(buf, sizeof buf, fp) != NULL) {
+	spt_ids[L_HOST   ] = sp_init();
+	while (1) {
+
+	  if (fgets(buf, bufsize-1, fp) == NULL)
+	    break;
+
 	  switch (list) {
 	  case L_CTLFILE:
 	    /* decid:\tfile\tnaddr; off1[,off2,...][\t#message] */
-	    if (!isdigit(buf[0]) && EQNSTR(buf, names[L_CHANNEL])) {
-	      list = L_CHANNEL;
-	      break;
+	    if (!isdigit(buf[0])) {
+	      if (EQNSTR(buf, names[L_CHANNEL])) {
+		list = L_CHANNEL;
+		break;
+	      }
+	      if (EQNSTR(buf, names[L_END])) {
+		return 1;
+	      }
 	    }
 	    if (!isdigit(buf[0]) ||
 		(cp = strchr(buf, ':')) == NULL) {
@@ -633,7 +661,8 @@ parse(fp)
 	      cfp->id = 0;
 	      cfp->mid = strsave(ocp);
 	      cfp->mark = 0;
-	      sp_install(symbol((u_char *)ocp), (u_char *)cfp, 0, spt_ids[L_CTLFILE]);
+	      sp_install(symbol((u_char *)ocp),
+			 (u_char *)cfp, 0, spt_ids[L_CTLFILE]);
 	    }
 	    while (isascii(*cp) && isspace(*cp))
 	      ++cp;
@@ -678,7 +707,6 @@ parse(fp)
 	    v->orig[L_CTLFILE] = v->orig[L_CHANNEL] = v->orig[L_HOST] = NULL;
 	    v->next[L_CHANNEL] = v->next[L_HOST] = NULL;
 	    v->prev[L_CHANNEL] = v->prev[L_HOST] = NULL;
-	    v->nextmark = NULL;
 	    sp_install(key, (u_char *)v, 0, spt_ids[L_VERTEX]);
 	    break;
 	  case L_CHANNEL:
@@ -687,8 +715,14 @@ parse(fp)
 	      list = L_HOST;
 	      break;
 	    }
+	    if (EQNSTR(buf, names[L_END])) {
+	      return 1;
+	    }
 	    /* FALL THROUGH */
 	  case L_HOST:
+	    if (EQNSTR(buf, names[L_END])) {
+	      return 1;
+	    }
 	    cp = buf-1;
 	    do {
 	      cp = strchr(cp+1, ':');
@@ -708,7 +742,6 @@ parse(fp)
 	      w->name = strsave(buf);
 	      w->kids = 0;
 	      w->link = w->lastlink = NULL;
-	      w->fifohead = w->fifotail = NULL;
 	      sp_install(symbol((u_char *)buf), (u_char *)w, 0, spt_ids[list]);
 	    }
 	    while (isascii(*cp) && isspace(*cp))
@@ -737,6 +770,8 @@ parse(fp)
 	      }
 	    }
 	    break;
+	default:
+	    break;
 	  }
 	}
 	return 1;
@@ -750,7 +785,7 @@ repscan(spl)
 {
 	register struct vertex *v, *vv;
 	struct web *w;
-	int fd, flag;
+	int fd, flag = 0;
 	struct stat stbuf;
 	long filecnt, filesizesum;
 
@@ -818,15 +853,15 @@ repscan(spl)
 	    v->ngroup = 0;
 	  }
 	  if (summary == 1 && !onlyuser) {
-	    printf("\t  %d file%s, ", filecnt,filecnt>1 ? "s":"");
+	    printf("\t  %d file%s, ", (int)filecnt, filecnt>1 ? "s":"");
 	    if (filesizesum == 0)
 	      printf("no file size info available\n");
 	    else
-	      printf("%d bytes total, %d bytes average\n",
-		     filesizesum,filesizesum/filecnt);
+	      printf("%ld bytes total, %d bytes average\n",
+		     filesizesum, (int)(filesizesum/filecnt));
 	  }
 	  if (summary > 1 && !onlyuser) {
-	    printf("\t  %d file%s\n", filecnt,filecnt>1 ? "s":"");
+	    printf("\t  %d file%s\n", (int)filecnt, filecnt>1 ? "s":"");
 	  }
 	}
 	return 0;
@@ -838,6 +873,13 @@ report(fp)
 {
 	if (!parse(fp))
 	  return;
+	if (schedq) {
+	  int c;
+	  while ((c = getc(fp)) != EOF) {
+	    putc(c,stdout);
+	  }
+	  return;
+	}
 
 	r_i = 0;
 	sp_scan(repscan, (struct spblk *)NULL, spt_ids[L_CHANNEL]);
@@ -873,13 +915,13 @@ printaddrs(v)
 	  }
 	  v->cfp->contents = malloc((u_int)stbuf.st_size);
 	  if (v->cfp->contents == NULL) {
-	    printf("\t\tmalloc(%d): out of memory!\n", stbuf.st_size);
+	    printf("\t\tmalloc(%d): out of memory!\n", (int)stbuf.st_size);
 	    close(fd);
 	    return;
 	  }
 	  errno = 0;
 	  if (read(fd, v->cfp->contents, stbuf.st_size) < stbuf.st_size){
-	    printf("\t\tread(%d): %s\n",stbuf.st_size,
+	    printf("\t\tread(%d): %s\n", (int)stbuf.st_size,
 		   errno == 0 ? "failed" : strerror(errno));
 	    close(fd);
 	    return;
@@ -921,25 +963,25 @@ printaddrs(v)
 	if (verbose > 1 && v->cfp->offset[0] > 0) {
 	  long dt = now - v->cfp->ctime;
 	  int fields = 2;
-	  printf(", %d bytes, age ", v->cfp->offset[0]);
+	  printf(", %ld bytes, age ", v->cfp->offset[0]);
 	  /* age (now-ctime) printout */
 	  if (dt > (24*3600)) {	/* Days */
-	    printf("%dd",dt /(24*3600));
+	    printf("%dd", (int)(dt /(24*3600)));
 	    dt %= (24*3600);
 	    --fields;
 	  }
 	  if (dt > 3600) {
-	    printf("%dh",dt/3600);
+	    printf("%dh",(int)(dt/3600));
 	    dt %= 3600;
 	    --fields;
 	  }
 	  if (dt > 60 && fields > 0) {
-	    printf("%dm",dt/60);
+	    printf("%dm",(int)(dt/60));
 	    dt %= 60;
 	    --fields;
 	  }
 	  if (fields > 0) {
-	    printf("%ds",dt);
+	    printf("%ds",(int)dt);
 	  }
 	}
 	putchar('\n');
