@@ -2,6 +2,10 @@
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
  */
+/*
+ *	Lots of modifications (new guts, more or less..) by
+ *	Matti Aarnio <mea@nic.funet.fi>  (copyright) 1992-1995
+ */
 
 #include "hostenv.h"
 #include <stdio.h>
@@ -17,9 +21,8 @@
 #include "ta.h"
 #include NDIR_H
 
-extern struct ctlfile *slurp();
-extern char *mailshare, *progname;
-/* extern int lockaddr(); */
+#include "prototypes.h"
+
 extern char *strchr();
 extern char *strnsave();
 
@@ -37,6 +40,7 @@ extern char *strnsave();
 struct not {
 	char *not;
 	char *dsn;
+	char *rcpt;
 };
 
 static void	/* There is notaryreport() on transporters also.. */
@@ -44,7 +48,9 @@ scnotaryreport(errfp,notary)
 	FILE *errfp;
 	struct not *notary;
 {
-	char *rcpt, *action, *status;
+	char *rcpt, *action, *status, *diagnostic, *wtt;
+
+	/* NOTARY: addres / action / status / diagnostic / wtt */
 
 	if (notary == NULL) {
 	  return; /* XX: ?? call into here should not happen.. */
@@ -58,15 +64,27 @@ scnotaryreport(errfp,notary)
 	*action++ = 0;
 	status   = strchr(action,'\001');
 	*status++ = 0;
+	diagnostic = strchr(status,'\001');
+	*diagnostic++ = 0;
+	wtt      = strchr(diagnostic,'\001');
+	*wtt++    = 0;
 
-	fprintf(errfp, "Rcpt: %s\n", rcpt);
+	if (strchr(rcpt,'@') != NULL)
+	  fprintf(errfp, "Final-Recipient: RFC822; %s\n", rcpt);
+	else
+	  fprintf(errfp, "Final-Recipient: X-LOCAL; %s\n", rcpt);
 	fprintf(errfp, "Action: %s\n", action);
 	fprintf(errfp, "Status: %s\n", status);
+	fprintf(errfp, "Diagnostic-Code: %s\n", diagnostic);
 	if (notary->dsn)
 	  fprintf(errfp, "X-DSN: %s\n", notary->dsn);
+	if (wtt[0] != 0)
+	  fprintf(errfp, "Remote-MTA: %s\n", wtt);
 	fprintf(errfp, "\n");
 	action[-1] = '\001';
 	status[-1] = '\001';
+	diagnostic[-1] = '\001';
+	wtt[-1]    = '\001';
 }
 
 
@@ -93,9 +111,10 @@ msgerror(vp, offset, message)
 	  return;
 	}
 	vp->cfp->haderror = 1;
-	fprintf(fp, "%c%c%ld:%ld:%ld\t%s\t%s\n",
+	fprintf(fp, "%c%c%ld:%ld:%ld:%ld\t%s\t%s\n",
 		_CF_DIAGNOSTIC, _CFTAG_NORMAL, offset,
 		vp->headeroffset, vp->drptoffset,
+		vp->dnsrecipient ? (vp->dnsrecipient - vp->cfp->contents) : -1,
 		notary, message);
 	fclose(fp);
 }
@@ -106,7 +125,7 @@ void
 reporterrs(cfpi)
 	struct ctlfile *cfpi;
 {
-	int i, n, wroteheader, byteidx, headeridx, drptidx, fd;
+	int i, n, wroteheader, byteidx, headeridx = -1, drptidx, fd;
 	long *lp;
 	char path[MAXPATHLEN], buf[BUFSIZ], *midbuf, *cp, *s, *eaddr;
 	char *deliveryform;
@@ -117,6 +136,7 @@ reporterrs(cfpi)
 	char *envid;
 	int notarycnt = 0;
 	int notaryspc = 0;
+	char *dnsrecipient = NULL;
 	struct ctlfile *cfp;
 
 	if (cfpi->haderror == 0)
@@ -193,7 +213,7 @@ reporterrs(cfpi)
 	  }
 	  
 	  /* Line fmt:
-	     <TAGS> offset ':' headeroffset <TAB> NOTARY-DATA <TAB> MESSAGE <NL> */
+	     <TAGS> offset ':' headeroffset ':' drptidx <TAB> NOTARY-DATA <TAB> MESSAGE <NL> */
 
 	  cp = cfp->contents + *lp + 2;
 	  byteidx = atoi(cp);
@@ -213,11 +233,18 @@ reporterrs(cfpi)
 	    while (isascii(*cp) && (isdigit(*cp) || *cp == '-'))
 	      ++cp;
 	  }
+	  if (*cp == ':') {
+	    int offs = atoi(++cp);
+	    if (offs > 0)
+	      dnsrecipient = cfp->contents + offs;
+	    while (isascii(*cp) && (isdigit(*cp) || *cp == '-'))
+	      ++cp;
+	  }
 	  notary = NULL;
 	  if (*cp == '\t') {
 	    notary = ++cp;
 	    while (*cp && *cp != '\t') ++cp;
-	    notary = strnsave(notary, cp-notary);
+	    notary = strnsave(notary, cp-notary); /* Make a copy of it */
 	    ++cp;
 	  }
 	  if (notary != NULL && *notary != 0) {
@@ -232,6 +259,7 @@ reporterrs(cfpi)
 	    }
 	    notaries[notarycnt].not = notary;
 	    notaries[notarycnt].dsn = NULL;
+	    notaries[notarycnt].rcpt = dnsrecipient;
 	    if (drptidx > 0) {
 	      notaries[notarycnt].dsn = cfp->contents + drptidx;
 	    }
@@ -265,13 +293,14 @@ reporterrs(cfpi)
 	fprintf(errfp, "--%s\n", boundarystr);
 	fprintf(errfp, "Content-Type: message/delivery-status\n\n");
 
-	fprintf(errfp, "Original-MTS-Type: INET\n");
+	if (mydomain() != NULL) {
+	  fprintf(errfp, "Reporting-MTA: dns; %s\n", mydomain() );
+	} else {
+	  fprintf(errfp, "Reporting-MTA: x-local-hostname; -unknown-\n");
+	}
 	if (envid != NULL)
 	  fprintf(errfp, "Original-Envelope-Id: %s\n",envid);
-	fprintf(errfp, "Final-MTS-Type: INET\n");
-	if (mydomain() != NULL) {
-	  fprintf(errfp, "Final-MTA: %s\n", mydomain() );
-	}
+	fprintf(errfp, "Arrival-Date: %s\n", rfc822date(&cfpi->ctime));
 	fprintf(errfp, "\n");
 
 	/* Now scan 'em all again for IETF-NOTARY */
