@@ -27,6 +27,7 @@ struct thread      *thread_tail = NULL;
 
 static struct threadgroup *thrg_root   = NULL;
 int idleprocs = 0;
+extern int global_wrkcnt;
 
 struct threadgroup *
 create_threadgroup(cep, wc, wh, withhost)
@@ -183,9 +184,10 @@ struct vertex *vtx;
 	return thr;
 }
 
-void
-delete_thread(thr)
+static void
+delete_thread(thr, ok)
 struct thread *thr;
+int ok;
 {
 	/* Unlink this thread from thread-chain, and thread
 	   group.  Decrement thread-group count		    */
@@ -280,7 +282,7 @@ struct thread *thr;
 		thg->idleproc = proc->next;
 		thg->idlecnt -= 1;
 		--idleprocs;
-		pick_next_vertex(proc);
+		pick_next_vertex(proc, ok);
 		break;
 	      }
 	    }
@@ -528,8 +530,9 @@ struct vertex *vtx;
  * Detach the vertex from its chains
  */
 void
-web_disentangle(vp)
+web_disentangle(vp, ok)
 	struct vertex *vp;
+	int ok;
 {
 	/* If it was in processing, remove process node binding.
 	   We do this only when we have reaped the channel program. */
@@ -541,7 +544,7 @@ web_disentangle(vp)
 	       if (vp->proc->vertex != NULL)
 	       vp->proc->vertex->proc = vp->proc;
 	     */
-	    pick_next_vertex(vp->proc);
+	    pick_next_vertex(vp->proc, ok);
 	  }
 	  vp->proc = NULL;
 	}
@@ -551,7 +554,7 @@ web_disentangle(vp)
 	/* The thread can now be EMPTY! */
 
 	if (vp->thread->vertices == NULL)
-	  delete_thread(vp->thread);
+	  delete_thread(vp->thread, ok);
 }
 
 void
@@ -574,17 +577,16 @@ struct thread *thr;
 	  ur_arr = (struct vertex **)
 	    emalloc(ur_size * sizeof (struct vertex *));
 	}
-	/* 3) Store the vertices into a re-arrange array */
-	for (i = 0, vp = thr->vertices; vp != NULL; ++i, vp = vp->nextitem) {
-	  if (i >= ur_size) {
+	/* 3) Store the vertices into a re-arrange array (and count) */
+	for (n = 0, vp = thr->vertices; vp != NULL; vp = vp->nextitem) {
+	  if (n >= ur_size) {
 	    ur_size *= 2;
 	    ur_arr = (struct vertex **)realloc((char *)ur_arr,
 					       ur_size *
 					       sizeof (struct vertex *));
 	  }
-	  ur_arr[i] = vp;
+	  ur_arr[n++] = vp;
 	}
-	n = i; /* Save the count */
 	/* 4) re-arrange pointers */
 	for (i = 0; i < n; ++i) {
 	  ni = ranny(n-1);
@@ -735,14 +737,17 @@ struct thread *thr;
 /*
  * pick_next_vertex() -- pick next free to process vertex in this thread
  *   - vertex proc pointer is non-null
+ *   - if the "ok" is not set, pick next THREAD for vertices, else use this
+ *     thread..
  *   - if the thread exhausts, pick another thread (check our scheduling vars)
  *     if the thread has vertices, but they are marked (proc != NULL),
  *     reschedule the thread, and pick another thread (if exist)
  *   - if all of the threads exhaust, move the process into idle pool
  */
 void
-pick_next_vertex(proc)
+pick_next_vertex(proc, ok)
 struct procinfo *proc;
+int ok;
 {
 	struct thread      *thr, *thr0;
 	struct threadgroup *thg;
@@ -750,8 +755,8 @@ struct procinfo *proc;
 
 	thr  = proc->thread;
 
-if (verbose) printf("pick_next_vertex(proc->tofd=%d, thr=0x%lx, jobs=%d)\n",
-		    proc->tofd, (long)thr, thr ? thr->jobs : 0);
+if (verbose) printf("pick_next_vertex(proc->tofd=%d, thr=0x%lx, jobs=%d ok=%d)\n",
+		    proc->tofd, (long)thr, thr ? thr->jobs : 0, ok);
 
 	if (proc->pid < 0) {	/* "Jim, He is dead!"		*/
 	  if (proc->thread != NULL)
@@ -780,8 +785,9 @@ if (verbose) printf(" ... NONE, current one has not been fed..\n");
 	vtx  = thr->vertices;
 	/* proc->vertex->proc = NULL; */ /* Mark that we are busy.. */
 
-	/* Ok, if that one was/is busy/marked off, try next */
-	while (vtx) {
+	/* Ok, if that one was/is busy/marked off, AND we are ok to
+	   use vertices from the same thread: try next vertex */
+	while (ok && vtx) {
 
 	  /* Is the current one in processing ? */
 	  if (vtx->proc == NULL) {
@@ -832,6 +838,10 @@ if (verbose) printf(" ... thr=same vtx=0x%x\n",(long)vtx);
 	  proc->ho->kids += 1;
 	  thr0->proc = NULL; /* no longer in there .. */
 	  thr->proc = proc;
+	  thg->thread = thr; /* Move the connection point to this
+				thread so that at the next time we
+				pick some other thread when skipping
+				failed threads.. */
 if (verbose) printf(" ... thr=0x%x vtx=0x%x ch=%s ho=%s\n",
 		    (long)thr,(long)proc->vertex,
 		    proc->ch->name,proc->ho->name);
@@ -874,24 +884,27 @@ time_t retrytime;
 	/* if we are already scheduled for the future, don't reschedule */
 	if (vtx->wakeup > now) {
 	  if (verbose)
-	    printf("prescheduled\n");
+	    printf("...prescheduled\n");
 	  return;
 	} else if (vtx->wakeup < now-7200 /* more than 2h in history .. */ )
 	  vtx->wakeup = now;
 
 	if (vtx->ce.nretries <= 0) {
 	  if (verbose)
-	    printf("ce.retries = %d\n", vtx->ce.nretries);
+	    printf("...ce.retries = %d\n", vtx->ce.nretries);
 	  return;
 	}
 
-	if (vtx->retryindex >= vtx->ce.nretries)
-	  vtx->retryindex = ranny(vtx->ce.nretries-1);
+	if (thr->retryindex >= vtx->ce.nretries)
+	  thr->retryindex = ranny(vtx->ce.nretries-1);
 	
 	/*
 	 * clamp retry time to a predictable interval so we
 	 * eventually bunch up deliveries.
 	 */
+	if (retrytime < now+63)
+	  vtx->wakeup = now;
+#if 0
 	skew = vtx->wakeup % vtx->ce.interval;
 	if (skew <= vtx->ce.interval / 2)
 	  skew = - (skew + (vtx->ce.skew - 1));
@@ -900,8 +913,15 @@ time_t retrytime;
 	skew = skew / vtx->ce.skew; /* want int div truncation */
 	
 	vtx->wakeup += (skew +
-			vtx->ce.retries[vtx->retryindex] * vtx->ce.interval);
-	vtx->retryindex++;
+			vtx->ce.retries[thr->retryindex] * vtx->ce.interval);
+#else
+	/* Actually we do NOT want to have synchronization of threads,
+	   as such causes simultaneous start of transporters, which
+	   causes "somewhat" spiky load behaviour */
+
+	vtx->wakeup += vtx->ce.retries[thr->retryindex] * vtx->ce.interval;
+#endif
+	thr->retryindex++;
 
 	/* XX: change this to a mod expression */
 	if (vtx->wakeup < now)
@@ -910,6 +930,9 @@ time_t retrytime;
 
 	wakeup = vtx->wakeup;
 
+	if (retrytime < now+63)
+	  retrytime = wakeup;
+
 	/* Time to expire ? */
 	if (vtx->ce.expiry > 0 && vtx->ce.expiry <= now && vtx->attempts > 0) {
 	  /* ... and now expire it! */
@@ -991,8 +1014,8 @@ reschedule(vp, factor, index)
 	}
 #endif
 	if (factor == -1) {
-	  if (vp->retryindex >= vp->ce.nretries)
-	    vp->retryindex = ranny(vp->ce.nretries-1);
+	  if (thr->retryindex >= vp->ce.nretries)
+	    thr->retryindex = ranny(vp->ce.nretries-1);
 
 	  /*
 	   * clamp retry time to a predictable interval so we
@@ -1006,8 +1029,8 @@ reschedule(vp, factor, index)
 	  skew = skew / vp->ce.skew; /* want int div truncation */
 
 	  vp->wakeup += (skew +
-			 vp->ce.retries[vp->retryindex] * vp->ce.interval);
-	  vp->retryindex++;
+			 vp->ce.retries[thr->retryindex] * vp->ce.interval);
+	  thr->retryindex++;
 	} else if (factor < -1) {
 	  vp->wakeup = -factor;
 	} else
@@ -1184,7 +1207,7 @@ FILE *fp;
 	  for (thr = thg->thread; thr && (thr_once || thr != thg->thread); thr = thr->nextthg) {
 	    thr_once = 0;
 
-	    fprintf(fp,"\t%s/%s/%d\tjobs=%d",
+	    fprintf(fp,"\t%s/%s/%d\trcpts=%d",
 		    /* thr->vertices->orig[L_CHANNEL]->name, */ thr->channel,
 		    /* thr->vertices->orig[L_HOST]->name, */ thr->host,
 		    thr->thgrp->withhost,thr->jobs);
@@ -1194,8 +1217,7 @@ FILE *fp;
 		thr->proc->thread == thr) {
 	      ++procs;
 	      fprintf(fp," pid=%d",thr->proc->pid);
-	    }
-	    if (thr->wakeup > now)
+	    } else if (thr->wakeup > now)
 	      fprintf(fp," wakeup=%d",(int)(thr->wakeup - now));
 	    if (thr->vertices && thr->vertices->ce.pending)
 	      fprintf(fp,
@@ -1205,7 +1227,7 @@ FILE *fp;
 	  }
 	  fprintf(fp,"\t\tThreads: %d/%d ",thg->threads,cnt);
 	  fprintf(fp,"Procs: %d/%d ",thg->transporters,procs);
-	  fprintf(fp,"Jobs: %d ",jobsum);
+	  fprintf(fp,"Rcpts: %d ",jobsum);
 	  jobtotal += jobsum;
 	  cnt = 0;
 	  for (;p != 0; p = p->next) ++cnt;
@@ -1213,6 +1235,6 @@ FILE *fp;
 	}
 	*timebuf = 0;
 	saytime((long)(now - sched_starttime), timebuf, 1);
-	fprintf(fp,"NumKids: %d   Idleprocs: %d   Jobs: %d   Uptime: %s\n",
-		numkids, idleprocs, jobtotal, timebuf);
+	fprintf(fp,"NumKids: %d   Idleprocs: %d   Messages: %d   Recipients: %d   Uptime: %s\n",
+		numkids, idleprocs, global_wrkcnt, jobtotal, timebuf);
 }
