@@ -167,6 +167,7 @@ char	rhostname[MAXHOSTNAMELEN+1];
 char	ihostaddr[3*4+1*3+2+1 /* == 18 for '[255.255.255.255]' */ +10/*slack*/];
 int	rport;
 int	skeptical  = 1;
+int	checkhelo  = 0;
 int	verbose    = 0;
 int	daemon_flg = 1;
 int	pid, routerpid;
@@ -229,7 +230,8 @@ static SIGNAL_TYPE timedout __((int sig));
 extern void detach(), prversion();
 static void smtpserver __((int insecure));
 static void setrhostname __((struct sockaddr_in *));
-extern void killprevious(), settrusteduser();
+extern int  killprevious();
+extern void settrusteduser();
 static void killr __((int pid));
 extern char *dottedquad();
 extern char *Version;
@@ -323,7 +325,7 @@ main(argc, argv)
 	  exit(1);
 	}
 	/* optarg = NULL; */
-	while ((c = getopt(argc, argv, "aBigl:np:L:M:P:R:s:Vv")) != EOF) {
+	while ((c = getopt(argc, argv, "aBighl:np:L:M:P:R:s:Vv")) != EOF) {
 	  switch (c) {
 #ifdef	USE_AUTH
 	    case 'a':
@@ -347,6 +349,9 @@ main(argc, argv)
 	    case 'g':	/* gullible */
 		skeptical = 0;
 		break;
+	    case 'h':	/* checkhelo */
+		checkhelo = 1;
+		break;
 	    case 'l':	/* log file */
 		logfile = strdup(optarg);
 		break;
@@ -476,7 +481,12 @@ main(argc, argv)
 	    if (postoffice == NULL
 		&& (postoffice = getzenv("POSTOFFICE")) == NULL)
 	      postoffice = POSTOFFICE;
-	    killprevious(SIGTERM, PID_SMTPSERVER);
+	    if (killprevious(SIGTERM, PID_SMTPSERVER) != 0) {
+	      fprintf(stderr,
+		      "%s: Can't write my pidfile!  Disk full ?\n",
+		      progname);
+	      exit(2);
+	    }
 	    if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
 	      fprintf(stderr,
 		      "%s: socket(AF_INET, SOCK_STREAM): %s\n",
@@ -785,7 +795,7 @@ char *cp;
 	 * tokenizer -- RFC821SCN.C, lets use it :)
 	 * We need it for proper handling of ESMTP anyway
 	 */
-	if (skeptical && partridge(helobuf, cfinfo)) {
+	if (checkhelo && skeptical && partridge(helobuf, cfinfo)) {
 	  rfc821_error_ptr += (cp - helobuf);
 	  type821err(501, buf,
 		     "Invalid `%s' parameter!  Err: %s",
@@ -804,13 +814,15 @@ char *cp;
 	if (skeptical && ihostaddr[0] != '\0'
 	    && rhostname[0] != '\0' && rhostname[0] != '['
 	    && cistrcmp(helobuf, rhostname) != 0) {
-	  type(-250, " %s That hostname is inconsistent with",
-	       myhostname);
-	  type(-250, " %s your address to name mapping.",
-	       myhostname);
+	  if (checkhelo) {
+	    type(-250, " %s That hostname is inconsistent with",
+	         myhostname);
+	    type(-250, " %s your address to name mapping.",
+	         myhostname);
+	  }
 	  type(carp->cmd==Hello2 ? -250: 250,
 	       " %s expected \"%s %s\"", myhostname,
-	       carp->verb, rhostname);
+	     carp->verb, rhostname);
 	} else
 	  type(carp->cmd==Hello2 ? -250: 250,
 	       " %s Hello %s", myhostname, helobuf);
