@@ -212,6 +212,8 @@ main(argc, argv)
 	    process(dp);
 	    ctlclose(dp);
 	  }
+break; /* XXX: Unfortunately, hold appears to hung somewhat, when doing
+		multiple accesses.. */
 	}
 	exit(0);
 	/* NOTREACHED */
@@ -241,22 +243,26 @@ process(dp)
 	  } else {
 	    char *action = NULL;
 	    char *status = NULL;
+	    char *diagnostics = NULL;
 	    switch (rp->status) {
 	    case EX_PROTOCOL:
 	    case EX_SOFTWARE:
 	      action = "failed";
-	      status = "500 (Protocol failure inside ZMAILER! AARGH!)";
+	      status = "5.0.0 (ZMailer internal protocol error)";
+	      diagnostics = "500 (Protocol failure inside ZMAILER! AARGH!)";
 	      break;
 	    case EX_TEMPFAIL:
 	      action = "delayed";
-	      status = "466 (Temporary routing lookup failure)";
+	      status = "4.4.3 (Temporary routing lookup failure)";
+	      diagnostics = "466 (Temporary routing lookup failure)";
 	      break;
 	    default:
 	      action = "failed";
-	      status = "500 (Unknown HOLD return code **CAN'T HAPPEN**)";
+	      status = "4.4.0 (Unknown ZMailer HOLD status ** CAN'T HAPPEN)";
+	      diagnostics = "500 (Unknown HOLD return code **CAN'T HAPPEN**)";
 	      break;
 	    }
-	    notaryreport(rp->addr->user,action,status);
+	    notaryreport(rp->addr->user,action,status,diagnostics);
 	    diagnostic(rp, rp->status, 0, "%s", rp->addr->user);
 	  }
 	}
@@ -275,6 +281,7 @@ process(dp)
 	  for (rp = dp->recipients; rp != NULL; rp = rp->next)
 	    if (rp->status == EX_OK) {
 	      notaryreport(rp->addr->user,"delayed",
+			   "4.3.1 (System spool full?)",
 			   "400 (Cannot resubmit anything, out of spool space?)");
 	      diagnostic(rp, EX_TEMPFAIL, 0,
 			 "cannot resubmit anything!");
@@ -344,7 +351,8 @@ process(dp)
 	for (rp = dp->recipients; rp != NULL; rp = rp->next)
 	  if (rp->status == EX_OK) {
 	    notaryreport(rp->addr->user,"relayed",
-			 "651 (Relayed via deferral channel)");
+			 "2.2.0 (Relayed via deferral channel)",
+			 "250 (Relayed via deferral channel)");
 	    diagnostic(rp, code, 0, cp);
 	  }
 }
