@@ -209,8 +209,12 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 
 	static struct ctldesc d; /* ONLY ONE OPEN AT THE TIME! */
 
+	if (selectaddr == ctlsticky)
+	  ctlsticky(NULL,NULL); /* Reset the internal state.. */
+
 	memset(&d,0,sizeof(d));
 
+	d.msgfd = -1; /* Always the zero is not good for your health .. */
 	if ((d.ctlfd = open(file, O_RDWR, 0)) < 0) {
 	  char cwd[MAXPATHLEN], buf[MAXPATHLEN+MAXPATHLEN+100];
 	  int e = errno;	/* Save it over the getwd() */
