@@ -44,33 +44,46 @@ char *okstr;
 	  case EX_PROTOCOL:
 	  case EX_USAGE:
 	  default:
-	      sprintf(msgbuf,"*SW-ERROR*rc=%d*",rc);
+	      sprintf(msgbuf,"*SW-ERROR*not-act-rc=%d*",rc);
 	      return msgbuf;
 	}
 }
 
-void
-notaryreport(arg1,arg2,arg3)
-	char *arg1, *arg2, *arg3;
+static char *arg5 = NULL; /* NOTARY arg5: 'While-Talking-To -host' */
+notary_setwtt(host)
+char *host;
 {
-	char *fmt = "%s\001%s\001%s";
+	if (arg5) free(arg5);
+	arg5 = strdup(host);
+}
+
+void
+notaryreport(arg1,arg2,arg3,arg4)
+	char *arg1, *arg2, *arg3, *arg4;
+{
+	char *fmt = "%s\001%s\001%s\001%s\001%s";
 	int len;
-	static char *A1 = NULL, *A2 = NULL, *A3 = NULL;
+	static char *A1 = NULL, *A2 = NULL, *A3 = NULL, *A4 = NULL;
 
 	if (arg1) { if (A1) free(A1); A1 = strdup(arg1); }
 	if (arg2) { if (A2) free(A2); A2 = strdup(arg2); }
 	if (arg3) { if (A3) free(A3); A3 = strdup(arg3); }
+	if (arg4) { if (A4) free(A4); A4 = strdup(arg4); }
 
-	len = 4; /* "\001\001" */
+	len = 5; /* "\001\001\001\001" */
 	if (A1) len += strlen(A1);
 	if (A2) len += strlen(A2);
 	if (A3) len += strlen(A3);
+	if (A4) len += strlen(A4);
+	if (arg5) len += strlen(arg5);
 
 	if (!notarybuf)
 	  notarybuf = (char*) emalloc(len);
 	else
 	  notarybuf = (char*) erealloc(notarybuf,len);
-	sprintf(notarybuf,fmt,A1?A1:"",A2?A2:"",A3?A3:"");
+	sprintf(notarybuf,fmt,
+		A1?A1:"",A2?A2:"",A3?A3:"",A4?A4:"",
+		arg5?arg5:"");
 }
 
 /*VARARGS*/
