@@ -105,8 +105,7 @@ strqcpy(buf,buflen,str)
 }
 
 
-static int
-cvtspace_copy(rp)
+int cvtspace_copy(rp)
 struct rcpt *rp;
 {
 	int hdrcnt = 0;
