@@ -10,13 +10,16 @@
 #include <syslog.h>
 #endif
 
+extern int errno;
+extern char *progname;
+
 /*VARARGS*/
 void
 warning(fmt, arg, arg2, arg3, arg4)
 	char *fmt, *arg, *arg2, *arg3, *arg4;
 {
 	static int isopen = 0;
-	extern char *progname;
+	int e = errno; /* Save it over the openlog() .. */
 
 	if (progname != NULL)
 		(void) fprintf(stderr, "%s: ", progname);
@@ -31,9 +34,12 @@ warning(fmt, arg, arg2, arg3, arg4)
 #endif	/* LOG_MAIL */
 		isopen = 1;
 	}
+	errno = e;
 	syslog(LOG_ERR, fmt, arg, arg2, arg3, arg4);
 #else	/* SVID */
+#ifdef LINT
 	isopen = isopen;	/* appease lint */
+#endif
 #endif	/* BSD */
 }
  
