@@ -604,7 +604,9 @@ deliver(dp, rp, timestring)
 	case TO_PIPE:		/* pipe to program */
 	  /* one should disallow this if uid == nobody? */
 	  if (uid == nobody) {
-	    notaryreport("?program?", "failed", "550 (Mail to program disallowed w/o proper priviledges)");
+	    notaryreport("?program?", "failed",
+			 "5.2.1 (Mail to program disallowed w/o proper privileges)",
+			 "550 (Mail to program disallowed w/o proper privileges)");
 	    DIAGNOSTIC(rp, EX_UNAVAILABLE,
 		       "mail to program disallowed", 0);
 	    return;
@@ -618,12 +620,14 @@ deliver(dp, rp, timestring)
 	  if (uid == nobody) {
 	    if (propably_x400(rp->addr->user)) {
 	      notaryreport(rp->addr->user,"failed",
+			   "5.1.4 (this feels like a misplaced X.400 address -- no support for them)",
 			   "550 (this feels like a misplaced X.400 address -- no support for them)");
 	      DIAGNOSTIC(rp, EX_UNAVAILABLE,
 			 "This appears to be a misplaced X.400 address, no support for them", 0);
 	    } else {
 	      notaryreport(rp->addr->user,"failed",
-			   "550 (mail to file disallowed w/o priviledges)");
+			   "5.2.1 (Mail to file disallowed w/o proper privileges)",
+			   "550 (mail to file disallowed w/o proper privileges)");
 	      DIAGNOSTIC(rp, EX_UNAVAILABLE,
 			 "mail to file disallowed", 0);
 	    }
@@ -650,6 +654,8 @@ deliver(dp, rp, timestring)
 	     [mea@utu.fi] */
 	  ismbox = 1;
 	  if (at) *at = 0;
+	  setpwent(); /* Effectively rewind the database,
+			 needed for multi-recipient processing ? */
 	  if ((pw = getpwnam(rp->addr->user)) == NULL) {
 	    if (isascii(*(rp->addr->user))
 		&& isupper(*(rp->addr->user))) {
@@ -661,11 +667,14 @@ deliver(dp, rp, timestring)
 	      if (at) *at = '@';
 	      if (propably_x400(rp->addr->user)) {
 		notaryreport(rp->addr->user,"failed",
+			     "5.1.4 (this feels like a misplaced X.400 address -- no support for them)",
 			     "550 (this feels like a misplaced X.400 address -- no support for them)");
 		DIAGNOSTIC(rp, EX_UNAVAILABLE,
 			   "This appears to be a misplaced X.400 address, no support for them", 0);
 	      } else {
-		notaryreport(rp->addr->user,"failed","550 (User does not exist)");
+		notaryreport(rp->addr->user,"failed",
+			     "5.1.1 (User does not exist)",
+			     "550 (User does not exist)");
 		if (at) *at = 0;
 		DIAGNOSTIC(rp, EX_NOUSER,
 			   "user \"%s\" doesn't exist", rp->addr->user);
@@ -695,6 +704,7 @@ deliver(dp, rp, timestring)
 
 	    char *mailbox = getzenv("MAILBOX");
 	    notaryreport(rp->addr->user,"failed",
+			 "4.3.5 (System mailbox configuration is wrong, we are in deep trouble..)",
 			 "466 (System mailbox configuration is wrong!  No such directory!  Aargh!)");
 	    DIAGNOSTIC(rp, EX_TEMPFAIL,
 		       "System mailbox configuration is wrong!  No such directory (%s)!  Aargh!",maildirs[0]);
@@ -743,6 +753,7 @@ deliver(dp, rp, timestring)
 	}
 	if (exstat(rp, file, &st, lstat) < 0) {
 	  notaryreport(rp->addr->user,"failed",
+		       "4.2.0 (User's mailbox disappeared, will retry)",
 		       "466 (User's mailbox disappeared, will retry)");
 	  DIAGNOSTIC(rp, EX_TEMPFAIL,
 		     "mailbox file \"%s\" disappeared", file);
@@ -752,9 +763,10 @@ deliver(dp, rp, timestring)
 	
 	/* Solaris has "interesting" /dev/null -- it is a symlink
 	   to the actual device.. So lets just use that magic
-	   name and create a FAST write to  /dev/null..  */
+	   name and create a FAST "write" to  /dev/null..  */
 	if (strcmp(file,"/dev/null") == 0) {
-	  notaryreport(rp->addr->user,"delivered",
+	  notaryreport(rp->addr->user,"delivery",
+		       "2.2.0 (delivery successfull)",
 		       "250 (Delivered successfully)");
 	  DIAGNOSTIC(rp, EX_OK, (char *)NULL, 0);
 	  return;
@@ -762,7 +774,8 @@ deliver(dp, rp, timestring)
 
 	if (!S_ISREG(st.st_mode)) {
 	  /* XX: may want to deliver to named pipes */
-	  notaryreport("?special-file?","failed",
+	  notaryreport(rp->addr->user,"failed",
+		       "5.2.1 (Attempting to deliver into non-regular file)",
 		       "500 (Attempting to deliver into non-regular file)");
 	  DIAGNOSTIC(rp, EX_UNAVAILABLE,
 		     "attempted delivery to special file \"%s\"",
@@ -771,7 +784,8 @@ deliver(dp, rp, timestring)
 	}
 	
 	if (st.st_nlink > 1) {
-	  notaryreport("?file-links?","failed",
+	  notaryreport(rp->addr->user,"failed",
+		       "4.2.1 (Destination file ambiguous)",
 		       "500 (Destination file has more than one name..)");
 	  DIAGNOSTIC(rp, EX_UNAVAILABLE,
 		     "too many links to file \"%s\"", file);
@@ -805,7 +819,9 @@ deliver(dp, rp, timestring)
 	    rp->status = EX_NOPERM;
 	  else
 	    rp->status = EX_UNAVAILABLE;
-	  notaryreport(file,"failed","500 (File open for append failed)");
+	  notaryreport(file,"failed",
+		       "4.2.1 (File open for append failed)",
+		       "500 (File open for append failed)");
 	  DIAGNOSTIC(rp, rp->status, fmtbuf, file);
 	  setrootuid(rp);
 	  return;
@@ -826,7 +842,9 @@ deliver(dp, rp, timestring)
 	}
 	if (st.st_ino != s2.st_ino || st.st_dev != s2.st_dev ||
 	    s2.st_nlink != 1) {
-	    notaryreport(file,"delayed","450 (Lost race for mailbox)");
+	    notaryreport(file,"delayed",
+			 "4.4.5 (Lost race for mailbox file)",
+			 "450 (Lost race for mailbox file)");
 	    DIAGNOSTIC(rp, EX_TEMPFAIL,
 		       "lost race for mailbox \"%s\"", file);
 	    close(fdmail);
@@ -843,7 +861,9 @@ deliver(dp, rp, timestring)
 #ifdef	USE_MAILLOCK
 	  if (*(rp->addr->user) == TO_FILE) {
 	    if ((pw = getpwuid(uid)) == NULL) {
-	      notaryreport(file,"failed","510 (File file has no owner)");
+	      notaryreport(file,"failed",
+			   "5.2.1 (Target file has no known owner)",
+			   "510 (Target file has no known owner)");
 	      DIAGNOSTIC(rp, EX_NOPERM,
 			 "File \"%s\" has no owner", file);
 	      return;
@@ -862,7 +882,9 @@ deliver(dp, rp, timestring)
 	      cp = NULL;
 	      break;
 	    case L_MAXTRYS:
-	      notaryreport(file,"delayed","450 (File unlocking fails)");
+	      notaryreport(file,"delayed",
+			   "4.4.5 (Mailbox unlocking fails)",
+			   "450 (Mailbox unlocking fails)");
 	      DIAGNOSTIC(rp, EX_TEMPFAIL, "\"%s\" is locked", file);
 	      return;
 	    case L_NAMELEN:
@@ -920,6 +942,7 @@ deliver(dp, rp, timestring)
 	      dotunlock(file);
 #endif /* USE_DOTLOCK */
 	    notaryreport(file,"delayed",
+			 "4.4.5 (File locking with NFS lock failed)",
 			 "450 (File locking with NFS lock failed)");
 	    DIAGNOSTIC(rp, EX_TEMPFAIL,
 		       "nfs lock() can't lock \"%s\"", file);
@@ -942,6 +965,7 @@ deliver(dp, rp, timestring)
 	      dotunlock(file);
 #endif /* USE_DOTLOCK */
 	    notaryreport(file,"delayed",
+			 "4.4.5 (File locking with  lockf  failed)",
 			 "450 (File locking with  lockf  failed)");
 	    DIAGNOSTIC(rp, EX_TEMPFAIL,
 		       "lockf() can't lock \"%s\"", file);
@@ -963,6 +987,7 @@ deliver(dp, rp, timestring)
 	      dotunlock(file);
 #endif /* USE_DOTLOCK */
 	    notaryreport(file,"delayed",
+			 "4.4.5 (File locking with  flock  failed)",
 			 "450 (File locking with  flock  failed)");
 	    DIAGNOSTIC(rp, EX_TEMPFAIL,
 		       "flock() can't lock \"%s\"", file);
@@ -1008,7 +1033,8 @@ deliver(dp, rp, timestring)
 	setrootuid(rp);
 	if (fp != NULL) {
 	  fclose(fp);		/* this closes fdmail */
-	  notaryreport(rp->addr->user,"delivered",
+	  notaryreport(rp->addr->user,"delivery",
+		       "2.2.0 (Delivered successfully)",
 		       "250 (Delivered successfully)");
 	  DIAGNOSTIC(rp, EX_OK, (char *)NULL, 0);
 	}
@@ -1036,7 +1062,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	int lastch;
 
 	if ((fp = fdopen(fdmail, fdopmode)) == NULL) {
-	  notaryreport(NULL,NULL,NULL);
+	  notaryreport(NULL,NULL,NULL,NULL);
 	  DIAGNOSTIC3(rp, EX_TEMPFAIL, "cannot fdopen(%d,\"%s\")",
 		      fdmail,fdopmode);
 	  return NULL;
@@ -1087,7 +1113,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	    if (len > 0 && (fwrite("\n\n", 1, len, fp) != len
 			    || fflush(fp) == EOF)) {
 
-	      notaryreport(NULL,NULL,NULL);
+	      notaryreport(NULL,NULL,NULL,NULL);
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "cleansing of \"%s\" failed", file);
 	      /* XX: should I really do this? */
@@ -1121,6 +1147,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 		    FROM_,fromuser,timestring) == EOF
 	    || writeheaders(rp,fp,"\n",convert_qp,0) < 0) {
 	  notaryreport(rp->addr->link->user, "failed",
+		       "4.2.2 (Write to user's mailbox failed)",
 		       "500 (Write to user's mailbox failed)");
 	  DIAGNOSTIC(rp, EX_IOERR,
 		     "header write to \"%s\" failed", file);
@@ -1150,7 +1177,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	   fputc('\n', fp);
 	 */
 	if (fputc('\n', fp) == EOF || fflush(fp) == EOF) {
-	  notaryreport(NULL,NULL,NULL);
+	  notaryreport(NULL,NULL,NULL,NULL);
 	  DIAGNOSTIC3(rp, EX_IOERR,
 		      "message write to \"%s\" failed: %s",
 		      file, strerror(errno));
@@ -1247,13 +1274,13 @@ program(dp, rp, timestring)
 
 	/* now we can fork off and run the command... */
 	if (pipe(in) < 0) {
-	  notaryreport(NULL,NULL,NULL);
+	  notaryreport(NULL,NULL,NULL,NULL);
 	  DIAGNOSTIC(rp, EX_OSERR, "cannot create pipe from \"%s\"",
 		     rp->addr->user+1);
 	  return;
 	}
 	if (pipe(out) < 0) {
-	  notaryreport(NULL,NULL,NULL);
+	  notaryreport(NULL,NULL,NULL,NULL);
 	  DIAGNOSTIC(rp, EX_OSERR, "cannot create pipe to \"%s\"",
 		     rp->addr->user+1);
 	  close(in[0]);
@@ -1292,7 +1319,7 @@ program(dp, rp, timestring)
 	  write(2, "Cannot exec /bin/sh\n", 20);
 	  _exit(128);
 	} else if (pid < 0) {	/* fork failed */
-	  notaryreport(NULL,NULL,NULL);
+	  notaryreport(NULL,NULL,NULL,NULL);
 	  DIAGNOSTIC(rp, EX_OSERR, "cannot fork", 0);
 	  return;
 	} /* parent */
@@ -1355,7 +1382,7 @@ program(dp, rp, timestring)
 	} else
 		i = EX_OK;
 #endif	/* USE_UNIONWAIT */
-	notaryreport(NULL,NULL,NULL); /* XX: proper values ? */
+	notaryreport(NULL,NULL,NULL,NULL); /* XX: proper values ? */
 	DIAGNOSTIC(rp, i, "%s", buf);
 	return;
 }
@@ -1413,6 +1440,7 @@ creatembox(rp, filep, uid, gid, pw)
 	/* assert *filep != NULL */
 	if (fd == -1) {
 	  notaryreport(rp->addr->user, "failed",
+		       "4.3.1 (can't create user spool mailbox file)",
 		       "466 (can't create user spool mailbox file)");
 	  DIAGNOSTIC(rp, EX_CANTCREAT, "can't create \"%s\"", *filep);
 	}
@@ -1440,7 +1468,8 @@ createfile(rp, file, uid)
 	      sprintf(msg,"466 (*INVOCATION BUG* Can't create user spool mailbox file: \"%s\", Directory stat() error: %s)",
 		      file,strerror(errno));
 	      *cp = 0;
-	      notaryreport(rp->addr->user, "failed", msg);
+	      notaryreport(rp->addr->user, "failed",
+			   "4.3.5 (Something wrong, bad config ?)", msg);
 	      DIAGNOSTIC(rp, i, "stat failed on %s", file);
 	      return -2;
 	    }
@@ -1448,7 +1477,8 @@ createfile(rp, file, uid)
 	    if (st.st_mode & 020) { /* group writable? */
 	      if (!setupuidgid(rp, uid, st.st_gid)) {
 		notaryreport(rp->addr->user, "failed",
-			     "466 (*INVOCATION BUG* The mailbox directory is group writable, but can't change my uid to it)");
+			     "4.3.5 (The mailbox configuration is faulty)",
+			     "466 (*INVOCATION BUG* The mailbox directory is group writable, but can't change my gid to it)");
 		DIAGNOSTIC(rp, i,
 			   "failed changing group id to create file in \"%s\"",
 			   file);
@@ -1477,9 +1507,12 @@ createfile(rp, file, uid)
 	  i = EX_CANTCREAT;
 	/* convoluted to maintain 4 arguments to DIAGNOSTIC */
 	sprintf(msg, "466 (error [%s] creating \"%%s\")", strerror(saverrno));
-	notaryreport(rp->addr->user, "failed",msg);
+	notaryreport(rp->addr->user, "failed",
+		     "4.3.0 (Other mailsystem error)",
+		     msg);
 	sprintf(msg, "error (%s) creating \"%%s\"", strerror(saverrno));
 	DIAGNOSTIC(rp, i, msg, file);
+	errno = saverrno;
 	return -2;
 }
 
