@@ -74,18 +74,20 @@ struct maildesc {
 	char	*argv[20];
 };
 
-#define	MO_FFROMFLAG		   01
-#define	MO_RFROMFLAG		   02
-#define	MO_NORESETUID		   04
-#define	MO_STRIPQUOTES		  010
-#define	MO_MANYUSERS		  020
-#define	MO_RETURNPATH		  040
-#define	MO_UNIXFROM		 0100
-#define	MO_HIDDENDOT		 0200
-#define	MO_ESCAPEFROM		 0400
-#define	MO_STRIPHIBIT		01000
-#define	MO_REMOTEFROM		02000
-#define	MO_CRLF			04000
+#define	MO_FFROMFLAG		    01
+#define	MO_RFROMFLAG		    02
+#define	MO_NORESETUID		    04
+#define	MO_STRIPQUOTES		   010
+#define	MO_MANYUSERS		   020
+#define	MO_RETURNPATH		   040
+#define	MO_UNIXFROM		  0100
+#define	MO_HIDDENDOT		  0200 /* SMTP dot-duplication */
+#define	MO_ESCAPEFROM		  0400
+#define	MO_STRIPHIBIT		 01000
+#define	MO_REMOTEFROM		 02000
+#define	MO_CRLF			 04000
+#define MO_BSMTP		010000 /* BSMTP-wrapping -- with HIDDENDOT.. */
+#define MO_BESMTP		020000 /* Extended BSMTP -- SIZE+8BITMIME+? */
 
 struct exmapinfo {
 	int	origstatus;
@@ -184,7 +186,7 @@ main(argc, argv)
 	}
 	if (errflg || optind != argc - 1 || host == channel) {
 	  fprintf(stderr,
-		  "Usage: %s [-V][-v][-8 || -Q][-f cfgfile][-w maxwidth][-c channel || -h host] mailer\n", argv[0]);
+		  "Usage: %s [-V][-v][-8 | -Q][-f cfgfile][-w maxwidth][-c channel | -h host] mailer\n", argv[0]);
 	  exit(EX_USAGE);
 	}
 	mailer = argv[optind];
@@ -374,17 +376,25 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	av[i] = NULL;
 	/* now we can fork off and run the command... */
 	if (pipe(in) < 0) {
-	  for (rp = startrp; rp != endrp; rp = rp->next)
+	  for (rp = startrp; rp != endrp; rp = rp->next) {
+	    notaryreport(rp->addr->user,"failed",
+			 "4.3.0 (Out of system resources, pipe creation failed)",
+			 "400 (pipe creation error, out of system resources ?)");
 	    diagnostic(rp, EX_OSERR, 0,
 		       "cannot create pipe from \"%s\"",
 		       mp->command);
+	  }
 	  return;
 	}
 	if (pipe(out) < 0) {
-	  for (rp = startrp; rp != endrp; rp = rp->next)
+	  for (rp = startrp; rp != endrp; rp = rp->next) {
+	    notaryreport(rp->addr->user,"failed",
+			 "4.3.0 (Out of system resources, pipe creation failed)",
+			 "400 (pipe creation error, out of system resources ?)");
 	    diagnostic(rp, EX_OSERR, 0,
 		       "cannot create pipe to \"%s\"",
 		       mp->command);
+	  }
 	  return;
 	}
 	if ((pid = fork()) == 0) { /* child, run the command */
@@ -404,8 +414,12 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  execv(mp->command, av);
 	  _exit(254);
 	} else if (pid < 0) {	/* couldn't fork, complain */
-	  for (rp = startrp; rp != endrp; rp = rp->next)
+	  for (rp = startrp; rp != endrp; rp = rp->next) {
+	    notaryreport(rp->addr->user,"failed",
+			 "4.3.0 (Out of system resources, fork failed)",
+			 "400 (fork failure, out of system resources ?)");
 	    diagnostic(rp, EX_OSERR, 0, "cannot fork");
+	  }
 	  return;
 	}
 	close(out[0]);
@@ -507,8 +521,13 @@ deliver(dp, mp, startrp, endrp, verboselog)
 
 	/* append message body itself */
 	if ((i = appendlet(dp, mp, tafp, verboselog, convertmode)) != EX_OK) {
-	  for (rp = startrp; rp != endrp; rp = rp->next)
+	  for (rp = startrp; rp != endrp; rp = rp->next) {
+	    notaryreport(rp->addr->user,"failed",
+			 /* Could indicate: 4.3.1 - mail system full ?? */
+			 "4.3.0 (Write to target failed for some reason)",
+			 "400 (Write to target failed for some reason)");
 	    diagnostic(rp, i, 0, "write error");
+	  }
 	  /* just to make sure nothing will get delivered */
 	  kill(pid, SIGTERM);
 	  sleep(1);
@@ -572,6 +591,14 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  if (verboselog)
 	    fprintf(verboselog,"Diagnostic: %s\n",cp);
 	  for (rp = startrp; rp != endrp; rp = rp->next) {
+	    if (i == EX_OK)
+	      notaryreport(rp->addr->user, "relayed",
+			   "2.2.0",
+			   "250 Delivered");
+	    else
+	      notaryreport(rp->addr->user, "failed",
+			   "4.3.0",
+			   "430 Unknown error");
 	    diagnostic(rp, i, 0, "%s", buf);
 	  }
 	  /* XX: still need to deal with MO_STRIPQUOTES */
@@ -1008,6 +1035,8 @@ readsmcf(file, mailer)
 	  case 'n':	m.flags &= ~MO_UNIXFROM;	break;
 	  case 'r':	m.flags |= MO_RFROMFLAG;	break;
 	  case 's':	m.flags |= MO_STRIPQUOTES;	break;
+	  case 'b':	m.flags |= MO_BSMTP;		break;
+	  case 'B':	m.flags |= MO_BESMTP|MO_BSMTP;	break;
 	  case 'A':		/* arpanet-compatibility */
 	  case 'C':		/* canonicalize remote hostnames */
 	  case 'D':		/* this mailer wants a Date: line */
