@@ -202,6 +202,8 @@ int first_uid = 0;		/* Make the opening connect with the UID of the
 
 int D_alloc = 0;		/* Memory usage debug */
 
+static char FAILED[] = "failed";
+
 #ifdef	lint
 #undef	putc
 #define	putc	fputc
@@ -468,6 +470,8 @@ main(argc, argv)
 	  s = strchr(file,'\t');
 	  if (s != NULL) {
 	    *s++ = 0;
+	    /* If different target host, close the old connection.
+	       In theory we could use same host via MX, but...     */
 	    if (host && strcmp(s,host) != 0) {
 	      if (smtp_isopen) {
 		smtpstatus = smtpwrite(smtpfp, verboselog, 0, "QUIT");
@@ -516,16 +520,29 @@ main(argc, argv)
 	    smtpstatus = smtpopen(host, smtpfp, verboselog);
 	    if (smtpstatus == EX_OK)
 	      smtp_isopen = 1;
+	    else {
+	      /* SMTPOPEN() made diagnostics data -- we report.. */
+	      struct rcpt *rphead;
+	      /* NOTARY: addres / action / status / diagnostic */
+	      for (rphead = dp->recipients;
+		   rphead != NULL;
+		   rphead = rphead->next) {
+		notaryreport(rphead->addr->user,NULL,NULL,NULL);
+		diagnostic(rphead, smtpstatus, 60, "%s", remotemsg);
+	      }
+	    }
 	  } else
 	    smtpstatus = EX_OK; /* Already open, we have a new job file */
 
-	  smtpstatus = process(dp, smtpstatus, host, smtpfp, verboselog);
+	  if (smtpstatus == EX_OK)
+	    smtpstatus = process(dp, smtpstatus, host, smtpfp, verboselog);
 	  if (verboselog)
 	    fclose(verboselog);
 	  verboselog = NULL;
 
 	  ctlclose(dp);
-	}
+	} /* while (!getout) ... */
+
 	if (smtpstatus == EX_OK) {
 	  if (smtp_isopen)
 	    smtpstatus = smtpwrite(smtpfp, verboselog, 0, "QUIT");
@@ -568,7 +585,7 @@ process(dp, smtpstatus, host, smtpfp, verboselog)
 
 	for (rp = rphead = dp->recipients; rp != NULL; rp = rp->next) {
 	  if (rp->next == NULL
-	      || rp->addr->link != rp->next->addr->link
+	      || rp->addr->link   != rp->next->addr->link
 	      || rp->newmsgheader != rp->next->newmsgheader) {
 	    if (smtpstatus == EX_OK && openstatus == EX_OK) {
 	      if (logfp != NULL && !loggedid) {
@@ -587,7 +604,8 @@ process(dp, smtpstatus, host, smtpfp, verboselog)
 		if (openstatus == EX_OK)
 		  smtp_isopen = 1;
 	      }
-	      smtpstatus = deliver(dp, smtpfp, verboselog, rphead, rp->next);
+	      if (openstatus == EX_OK)
+		smtpstatus = deliver(dp, smtpfp, verboselog, rphead, rp->next);
 	      /*
 		 if (openstatus != EX_OK) {
 		 fclose(smtpfp[0]); smtpfp[0] = NULL;
@@ -600,22 +618,14 @@ process(dp, smtpstatus, host, smtpfp, verboselog)
 	      rphead = rp->next;
 	    } else {
 	      while (rphead != rp->next) {
-		if (openstatus == EX_OK) {
-		  /* SMTP open -- meaning (propably) that we got reject
-		     from the remote server */
-		  /* NOTARY: addres / action / status / diagnostic */
-		  notaryreport(rp->addr->user,"failed",
-			       "5.1.1 (Target user does not exist)",
-			       NULL);
-		  diagnostic(rphead, smtpstatus, 60, "%s", remotemsg);
-		} else {
-		  /* SMTP not open -- remote didn't respond */
-		  /* NOTARY: addres / action / status / diagnostic */
-		  notaryreport(rp->addr->user,"failed",
-			       "5.4.1 (No answer from remote)",
-			       NULL);
-		  diagnostic(rphead, smtpstatus, 60, "%s", remotemsg);
-		}
+		/* SMTP open -- meaning (propably) that we got reject
+		   from the remote server */
+		/* NOTARY: addres / action / status / diagnostic */
+		notaryreport(rp->addr->user,FAILED,
+			     "5.0.0 (Target status indeterminable)",
+			     NULL);
+		diagnostic(rphead, smtpstatus, 60, "%s", remotemsg);
+
 		rphead = rphead->next;
 	      }
 	    }
@@ -1838,7 +1848,7 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 		hostname, ipaddress, se);
 	if (statusreport)
 	  report("%s",remotemsg+4);
-	notaryreport(NULL,NULL,"4.4.1 (TCP/IP-connection failure)", remotemsg);
+	notaryreport(NULL,FAILED,"5.4.1 (TCP/IP-connection failure)", remotemsg);
 
 	if (conndebug)
 		fprintf(stderr, "%s\n", remotemsg);
@@ -1953,13 +1963,13 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	  if ((r = write(outfd, buf, len)) == -1) {
 	    if (gotalarm) {
 	      strcpy(remotemsg, "Timeout on cmd write");
-	      notaryreport(NULL,NULL,"4.4.2 (timeout on cmd write)",
+	      notaryreport(NULL,"failure","5.4.2 (timeout on cmd write)",
 			   "smtp; 500 (timeout on cmd write)");
 	    } else
 	      {
 		se = strerror(errno);
 		sprintf(remotemsg, "smtp; 500 (write to server error: %s)", se);
-		notaryreport(NULL,NULL,"4.4.2 (write to server, err)",remotemsg);
+		notaryreport(NULL,FAILED,"5.4.2 (write to server, err)",remotemsg);
 	      }
 	    if (verboselog)
 	      fprintf(verboselog,"%s\n",remotemsg);
@@ -1971,7 +1981,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	    return EX_TEMPFAIL;
 	  } else if (r != len) {
 	    sprintf(remotemsg, "smtp; 500 (SMTP cmd write failure: Only wrote %d of %d bytes!)", r, len);
-	    notaryreport(NULL,NULL,"4.4.2 (SMTP cmd partial write failure)",remotemsg);
+	    notaryreport(NULL,FAILED,"5.4.2 (SMTP cmd partial write failure)",remotemsg);
 	    if (verboselog)
 	      fprintf(verboselog,"%s\n",remotemsg);
 	    fclose(smtpfp[0]); smtpfp[0] = NULL;
@@ -2078,7 +2088,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 		sprintf(remotemsg,
 			"smtp; 466 (Timeout on SMTP responce read, Cmd: %s)",
 			strbuf);
-	      notaryreport(NULL,NULL,"4.4.2 (smtp transaction read timeout)",remotemsg);
+	      notaryreport(NULL,FAILED,"5.4.2 (smtp transaction read timeout)",remotemsg);
 	    } else {
 		se = strerror(errno);
 		if (strbuf == NULL)
@@ -2089,7 +2099,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 		  sprintf(remotemsg,
 			  "smtp; 500 (Error on SMTP responce read: %s, Cmd: %s)",
 			  se, strbuf);
-		notaryreport(NULL,NULL,"4.4.2 (smtp transaction read timeout)",remotemsg);
+		notaryreport(NULL,FAILED,"5.4.2 (smtp transaction read timeout)",remotemsg);
 	      }
 	    dflag = 0;
 	    firstmx = 0;
@@ -2104,7 +2114,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	  } else {
 	    sprintf(remotemsg, "smtp; 500 (Server hung up on us! Cmd: %s)",
 		    strbuf == NULL ? "(null cmd)" : strbuf);
-	    notaryreport(NULL,NULL,"4.4.2 (server hung-up on us)",remotemsg);
+	    notaryreport(NULL,FAILED,"5.4.2 (server hung-up on us)",remotemsg);
 	    dflag = 0;
 	    if (verboselog)
 	      fprintf(verboselog,"%s\n",remotemsg);
@@ -2122,7 +2132,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 
 	if (cp >= buf+sizeof buf) {
 	  strcpy(remotemsg,"smtp; 500 (SMTP Response overran input buffer!)");
-	  notaryreport(NULL,NULL,"5.5.0 (SMTP-responce overran input buffer!)",remotemsg);
+	  notaryreport(NULL,"X-BUG","5.5.0 (SMTP-responce overran input buffer!)",remotemsg);
 	  dflag = 0;
 	  if (verboselog)
 	    fprintf(verboselog,"%s\n",remotemsg);
@@ -2136,7 +2146,7 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	*--cp = '\0';	/* kill the LF */
 	if (cp - buf < 5) {
 	  sprintf(remotemsg, "smtp; 500 (SMTP response '%s' unexpected!)", buf);
-	  notaryreport(NULL,NULL,"5.5.0 (SMTP responce unexpected)",remotemsg);
+	  notaryreport(NULL,"X-BUG","5.5.0 (SMTP responce unexpected)",remotemsg);
 	  dflag = 0;
 	  if (verboselog)
 	    fprintf(verboselog,"%s\n",remotemsg);
@@ -2397,7 +2407,7 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 
 	if (depth > 3) {
 	  sprintf(remotemsg,"500 (DNS: Recursive CNAME on '%s')",host);
-	  notaryreport(NULL,NULL,"4.4.3 (Recursive DNS CNAME)",remotemsg);
+	  notaryreport(NULL,FAILED,"5.4.3 (Recursive DNS CNAME)",remotemsg);
 	  fprintf(stderr, "%s\n", remotemsg);
 	  return EX_NOHOST;
 	}
@@ -2409,14 +2419,14 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	  fprintf(stderr, "res_mkquery failed\n");
 	  sprintf(remotemsg,
 		  "smtp; 466 (Internal: res_mkquery failed on host: %s)",host);
-	  notaryreport(NULL,NULL,"4.4.3 (DNS-failure)",remotemsg);
+	  notaryreport(NULL,FAILED,"5.4.3 (DNS-failure)",remotemsg);
 	  return EX_SOFTWARE;
 	}
 	n = res_send((void*)&qbuf, qlen, (void*)&answer, sizeof answer);
 	if (n < 0) {
 	  sprintf(remotemsg,
 		  "smtp; 466 (Internal: res_send failed on host: %s)",host);
-	  notaryreport(NULL,NULL,"4.4.3 (DNS-failure)",remotemsg);
+	  notaryreport(NULL,FAILED,"5.4.3 (DNS-failure)",remotemsg);
 	  return EX_TEMPFAIL;
 	}
 	eom = (msgdata *)&answer + n;
@@ -2435,11 +2445,11 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	     * change to return EX_TEMPFAIL iff hp->aa == 0.
 	     */
 	    sprintf(remotemsg, "smtp; 500 (Bind: no such domain: %s)", host);
-	    notaryreport(NULL,NULL,"4.4.4 (DNS lookup report)",remotemsg);
+	    notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	    return EX_NOHOST;
 	  case SERVFAIL:
 	    sprintf(remotemsg, "smtp; 500 (Bind: server failure: %s)", host);
-	    notaryreport(NULL,NULL,"4.4.4 (DNS lookup report)",remotemsg);
+	    notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	    return EX_TEMPFAIL;
 #ifdef OLDJEEVES
 	    /*
@@ -2460,11 +2470,11 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	  case NOTIMP:
 	  case REFUSED:
 	    sprintf(remotemsg, "smtp; 500 (Bind: unsupported query: %s)", host);
-	    notaryreport(NULL,NULL,"4.4.4 (DNS lookup report)",remotemsg);
+	    notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	    return EX_NOPERM;
 	  }
 	  sprintf(remotemsg, "smtp; 500 (Bind: unknown error, MX info unavailable: %s)", host);
-	  notaryreport(NULL,NULL,"4.4.4 (DNS lookup report)",remotemsg);
+	  notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	  return EX_UNAVAILABLE;
 	}
 	nmx = 0;
@@ -2581,7 +2591,7 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	}
 	if (n == 0) {/* MX's exist, but their WKS's show no TCP smtp service */
 	  sprintf(remotemsg, "500 (Bind: MX host does not support SMTP: %s)", host);
-	  notaryreport(NULL,NULL,"4.4.4 (DNS lookup report)",remotemsg);
+	  notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	  return EX_UNAVAILABLE;
 	}
 	nmx = n;
